@@ -1,6 +1,7 @@
 # import libraries
 from scipy.constants import nano, kilo, milli, centi, liter, minute, day, hour
 from chempy.properties.water_density_tanaka_2001 import water_density
+from chemicals import periodic_table
 from pubchempy import get_compounds 
 from math import pi, exp, ceil
 from matplotlib import pyplot 
@@ -18,6 +19,10 @@ import re
 def sigfigs_conversion(num, sigfigs_in = 2):
     return round(num, sigfigs=sigfigs_in, notation = 'sci')
 
+elemental_masses = {}
+for element in periodic_table:
+    elemental_masses[element.symbol] = element.MW
+
 def time_determination(time):
     unit = 'seconds'
     if time > 600:
@@ -44,15 +49,16 @@ class ROSSPkg():
         self.verbose = verbose
         self.jupyter = jupyter
         self.plot_title = None
-        if operating_system == 'mac':
-            import phreeqpy.iphreeqc.phreeqc_dll as phreeqc_mod
-        elif operating_system == 'windows':
+        if operating_system == 'windows':
             import phreeqpy.iphreeqc.phreeqc_com as phreeqc_mod
+        elif operating_system == 'unix':
+            import phreeqpy.iphreeqc.phreeqc_dll as phreeqc_mod
         else:
             print(f'--> ERROR: The operating system {operating_system} is not supported.')
         self.phreeqc_mod = phreeqc_mod
         self.parameters['os'] =  operating_system
         self.parameters['root_path'] = os.path.join(os.path.dirname(__file__))
+        self.databases = [re.search('(\w+)(?=.json)', database).group() for database in glob('./databases/*.json')]
         
     def define_general(self, database_selection, simulation = 'scaling', domain = 'single', domain_phase = None, quantity_of_modules = 1, simulation_type = 'transport', simulation_title = None):
         '''Establish general conditions'''
@@ -69,7 +75,7 @@ class ROSSPkg():
 
         title_line = f'TITLE\t {simulation_title}'
         database_line = 'DATABASE {}'.format(self.parameters['database_path'])
-#         if self.parameters['os'] == 'mac':
+#         if self.parameters['os'] == 'unix':
 #             database_line = 'DATABASE C:{}'.format(self.parameters['database_path'])
         self.results['general_conditions'] = [database_line, title_line]
             
@@ -201,11 +207,8 @@ class ROSSPkg():
             print('permeate_removal_per_cell', self.parameters['permeate_moles_per_cell'])
             print('active_cm_squared_cell', (self.parameters['active_m2_cell'] / centi**2))
 
-    def reaction(self, permeate_approach = 'linear_permeate', permeate_efficiency = 1, head_loss = 0.89, final_cf = 2):
-        '''
-        Define the REACTION block
-            The head_loss parameter default of 11% is sourced from “Reverse osmosis desalination: Modeling and experiment” by Fraidenraich et al.; 2009.
-        '''        
+    def reaction(self, permeate_approach = 'linear_permeate', permeate_efficiency = 1, head_loss = 0.89, final_cf = 2, evaporation_steps = 15):     
+        '''Define the REACTION blocks'''
         # establish parameters
         self.parameters['permeate_approach'] = permeate_approach
         
@@ -274,7 +277,6 @@ class ROSSPkg():
                 self.variables[f'module {module}'] = {'cf': cf}
 
             if self.parameters['simulation_type'] == 'transport':
-                self.results['reaction_block'].append('\n')
                 for cell in range(1, self.parameters['cells_per_module']+1):
                     if self.parameters['domain'] == 'single':
                         cell_number = cell + self.parameters['cells_per_module'] * module
@@ -283,44 +285,35 @@ class ROSSPkg():
                         cell_number = cell + self.parameters['cells_per_module'] * (module + self.parameters['quantity_of_modules']) + 1
                         reaction_index = (cell + self.parameters['cells_per_module'] * module)-1
                         
-                    reaction_line = f'REACTION {cell_number}'
+                    reaction_line = f'\nREACTION {cell_number}'
                     if cell < self.parameters['cells_per_module']:
                         reaction_line += f'\n\tH2O -1; {reaction_parameters[reaction_index]}' 
                     elif cell == self.parameters['cells_per_module']:
                         reaction_line += f'''\n\tH2O -1; {reaction_parameters[reaction_index]}'''   
                            
                     self.results['reaction_block'].append(reaction_line)
+                    
+                # the calculated reaction parameters will be added and printed to a generated PHREEQC input file
+                final_solution_mass = moles_remaining * self.parameters['water_mw'] * milli  #kg water mass
+                if self.parameters['os'] == 'windows':
+                    self.results['reaction_block'].append('#%s' %(permeate_approach))
+                    if permeate_approach == 'linear permeate':
+                        self.results['reaction_block'].append(f'''
+                #Permeate efficiency parameter: {permeate_efficiency}
+                #Effluent relative pressure: {head_loss}''')
+
+                        self.results['reaction_block'].append(f'''    #Effluent module {module + 1}:
+                #Estimated CF: {sigfigs_conversion(cf, 4)}
+                #Estimated final water mass: {final_solution_mass}\n\n''')
 
             elif self.parameters['simulation_type'] == 'evaporation':
-                parameter_quantity = 15                          
-                recursive_asymptote_multiplier = 1.335449219     # ??? arbitrary assignment of kg of water in the simulation?
-                moles_removed = sum(reaction_parameters)
-                initial_evaporation_parameter = moles_removed / recursive_asymptote_multiplier
-                evaporation_reaction_parameters = ['0', initial_evaporation_parameter]  # ???
-                for parameter in range(1, parameter_quantity):
-                    evaporation_reaction_parameter = evaporation_reaction_parameters[parameter] * 1/4
-                    evaporation_reaction_parameters.append(evaporation_reaction_parameter)
-
                 # define the reaction block
-                reaction_line = 'REACTION 1'
+                reaction_line = '\nREACTION 1'
                 reaction_line += '\n\tH2O -1; '
-                reaction_line += ' '.join([str(x) for x in evaporation_reaction_parameters]) 
+                reaction_line += f'{moles_removed} in {evaporation_steps} step'
                 reaction_line += ';\nINCREMENTAL_REACTIONS \ttrue'
                 self.results['reaction_block'].append(reaction_line)
 
-            # the calculated reaction parameters will be added and printed to a generated PHREEQC input file
-            final_solution_mass = moles_remaining * self.parameters['water_mw'] * milli  #kg water mass
-            if self.parameters['os'] == 'windows':
-                self.results['reaction_block'].append('#%s' %(permeate_approach))
-                if permeate_approach == 'linear permeate':
-                    self.results['reaction_block'].append(f'''
-        #Permeate efficiency parameter: {permeate_efficiency}
-        #Effluent relative pressure: {head_loss}''')
-
-                self.results['reaction_block'].append(f'''    #Effluent module {module + 1}:
-        #Estimated CF: {sigfigs_conversion(cf, 4)}
-        #Estimated final water mass: {final_solution_mass}\n\n''')
-
             if self.verbose:
                 print(f'Effluent module {module + 1} CF: {cf}')
 
@@ -557,7 +550,7 @@ class ROSSPkg():
         self.results['selected_output_block'] = []
         self.results['selected_output_block'].extend((first_line, file_name_line, reaction_line, temperature_line, total_elements_line, saturation_indices_line, equilibrium_phases_line, ph_line, time_line, distance_line, simulation_line, high_precision_line, alkalinity_line, solution_line, charge_balance_line, ionic_strength_line, step_line, water_line))
         
-    def define_paths(self, simulation_name = None, input_path = None, output_path = None):
+    def define_paths(self, simulation_name = None, input_path = None, output_path = None, external_file = False):
         # define the simulation name 
         simulation_number = 1
         if simulation_name is None:
@@ -570,7 +563,12 @@ class ROSSPkg():
                     permeate_approach_name = 'LinCF'
                 simulation_name = '-'.join([re.sub(' ', '_', str(x)) for x in [datetime.date.today(), 'ROSSpy', self.parameters['water_selection'], self.parameters['simulation_type'], self.parameters['database_selection'], self.parameters['simulation'], self.parameters['simulation_perspective'], permeate_approach_name]])
             
-        while os.path.exists(simulation_name):
+        if input_path is None:
+            directory = os.getcwd()
+        else:
+            directory = os.path.dirname(input_path)
+            
+        while os.path.exists(os.path.join(directory, simulation_name)):
             simulation_number += 1
             simulation_name = re.sub('(\-\d+$)', '', simulation_name)
             simulation_name = '-'.join([simulation_name, str(simulation_number)])
@@ -578,29 +576,36 @@ class ROSSPkg():
         # define the simulation input path 
         if input_path is None:
             self.parameters['input_file_name'] = 'input.pqi'
-            working_directory = os.getcwd()
-            self.simulation_path = os.path.join(working_directory, simulation_name)
+            self.simulation_path = os.path.join(directory, simulation_name)
             os.mkdir(self.simulation_path)
             self.parameters['input_path'] = os.path.join(self.simulation_path, self.parameters['input_file_name'])
         else:
             self.parameters['input_path'] = input_path
+            self.simulation_path = os.path.join(directory, simulation_name)
+            os.mkdir(self.simulation_path)
             
         # define the simulation output path 
-        if output_path is None:
-            self.parameters['output_file_name'] = 'selected_output.pqo'
+        if external_file:
+            self.parameters['output_file_name'] = 'output.pqo'
             self.parameters['output_path'] = os.path.join(self.simulation_path, self.parameters['output_file_name'])
-        else:
-            self.parameters['output_path'] = output_path 
-            
+        else:            
+            if output_path is None:
+                self.parameters['output_file_name'] = 'selected_output.csv'
+                self.parameters['output_path'] = os.path.join(self.simulation_path, self.parameters['output_file_name'])
+            else:
+                self.parameters['output_path'] = output_path 
             
         return simulation_name, self.parameters['input_path'], self.parameters['output_path']
 
     def export(self, simulation_name = None, input_path = None, output_path = None, external_file = False):
         """View and export the PHREEQC input file"""    
-        simulation_name, input_path, output_path = self.define_paths(simulation_name, input_path, output_path)
+        simulation_name, input_path, output_path = self.define_paths(simulation_name, input_path, output_path, external_file)
             
         # comment the corresponding simulation in the input file
         simulation_line = f'# {self.simulation_path}'
+        if external_file:
+            with open(os.path.join(self.simulation_path, 'input.pqi'), 'w') as input:
+                input.write(self.input_file)
         if not external_file:
             self.results['solution_block'].insert(0, simulation_line)
             self.results['complete_lines'] = chain(self.results['general_conditions'], self.results['solution_block'], self.results['equilibrium_phases_block'], self.results['reaction_block'], self.results['selected_output_block'], self.results['transport_block']) 
@@ -650,18 +655,24 @@ class ROSSPkg():
         variables_table.to_csv(variables_path)
         
         # export the predicted effluent concentrations
-        effluent_concentrations = pandas.DataFrame(list(self.predicted_effluent.items()), columns = ['elements', 'ppm conc'])
+        effluent_concentrations = pandas.DataFrame(list(self.predicted_effluent.items()), columns = ['elements', 'conc (input units)'])
         effluent_concentrations.index = effluent_concentrations['elements']
         del effluent_concentrations['elements']
         effluent_path = os.path.join(self.simulation_path, 'effluent_predictions.csv')
         effluent_concentrations.to_csv(effluent_path)
         
-    def parse_input(self, input_file_path, simulation, water_selection = None, simulation_name = None, active_feed_area = None):        
+    def parse_input(self, input_file_path, simulation, water_selection = None, simulation_name = None, active_m2 = None):        
         # open the input file
         with open(input_file_path, 'r') as file:
             self.input_file = file.read()
         if self.verbose:
             print(self.input_file)
+           
+        # identify the database in the name
+        self.parameters['database_selection'] = 'pitzer'
+        for db in self.databases:
+            if re.search(db, input_file_path):
+                self.parameters['database_selection'] = db
             
         # parse the input_file       
         input_df = pandas.read_table(input_file_path, names = ['content'], sep='\n')
@@ -671,12 +682,19 @@ class ROSSPkg():
         self.parameters['water_selection'] = water_selection
         self.parameters['simulation'] = simulation
         self.parameters['domain_phase'] = False
+        self.predicted_effluent = {}
+        permeate_moles = 0
         for index, row in input_df.iteritems():
             if re.search('DATABASE', row):
                 row = re.sub('DATABASE ','',row)
                 self.parameters['database_selection'] = re.sub('\.dat', '', os.path.basename(row))
-                self.parameters['database_path'] = os.path.join(self.parameters['root_path'], 'databases','{}.dat'.format(self.parameters['database_selection']))
-                self.define_database()
+            elif re.search('(^[A-Z][a-z \(]?[\s\d])', row):
+                element = re.search('(^[A-Z][a-z]?)', row).group()
+                conc = re.search('(?<!\()([\d\.]+)', row).group(1)
+                if float(conc) > 0:
+                    self.predicted_effluent[element] = float(conc)
+            elif re.search('-water\s+\d', row):
+                water_mass = float(re.sub('(-water\s+)', '', row))
             elif re.search('linear_cf', row):
                 self.parameters['permeate_approach'] = 'linear_cf'
             elif re.search('EQUILIBRIUM_PHASES', row):
@@ -684,12 +702,16 @@ class ROSSPkg():
                 row = float(re.sub('(\-\d+)', '', row))
                 if row != 1:
                     self.parameters['domain_phase'] = 'Mobile'
+            elif re.search('H2O -1; ', row):
+                permeate_moles += float(re.sub('(H2O -1; )', '', row))
             elif re.search('TRANSPORT', row):
                 self.parameters['simulation_type'] = 'transport'
             elif re.search('-cells', row):
                 self.parameters['cells_per_module'] = float(re.sub('-cells\s+', '', row))
             elif re.search('-shifts', row):
                 self.simulation_shifts = float(re.sub('-shifts\s+', '', row))
+            elif re.search('-file', row):
+                self.selected_output_file_name = re.sub('-file\s+', '', row)
             elif re.search('-time_step', row):
                 self.parameters['timestep'] = float(re.sub('-time_step\s+', '', row).split('\t')[0])
                 self.parameters['simulation_time'] = self.parameters['timestep'] * self.simulation_shifts
@@ -705,19 +727,47 @@ class ROSSPkg():
                 elif row == 1:
                     self.parameters['simulation_perspective'] = 'all_time'
                     
+        # open the respective database
+        self.parameters['database_path'] = os.path.join(self.parameters['root_path'], 'databases','{}.dat'.format(self.parameters['database_selection']))
+        self.define_database()
+                    
+        # predict the effluent concentrations
+        water_molar_mass = 2*elemental_masses['H']+elemental_masses['O']
+        final_water_mass = water_mass - permeate_moles*water_molar_mass*milli
+        self.cumulative_cf = water_mass/final_water_mass
+        for element in self.predicted_effluent:
+            self.predicted_effluent[element] = self.predicted_effluent[element]*self.cumulative_cf
+                
         # define the simulation folder and parameters
         if self.parameters['domain_phase'] is not None:
             self.plot_title = '{} phase {} from the {} after {}'.format(self.parameters['domain_phase'], self.parameters['simulation'], self.parameters['water_selection'], sigfigs_conversion(self.parameters['simulation_time']))
-        if active_feed_area is None:
+        self.parameters['active_m2'] = active_m2
+        if self.parameters['active_m2'] is None:
             self.parameters['active_m2'] = 37
-            self.parameters['active_m2_cell'] = self.parameters['active_m2']/self.parameters['cells_per_module']
-        self.export(simulation_name, external_file = True)
+        self.parameters['active_m2_cell'] = self.parameters['active_m2']/self.parameters['cells_per_module']
+        self.export(simulation_name, input_file_path, external_file = True)
 
-    def execute(self, input_file_path = None, simulated_to_real_time = 9.29):
+    def execute(self, simulated_to_real_time = 9.29):
         '''Execute a PHREEQC input file '''
         def run(input_file, first=False):
             phreeqc = self.phreeqc_mod.IPhreeqc()  
-            phreeqc.load_database(self.parameters['database_path'])                
+            print(self.parameters['database_path'])
+            print(type(self.parameters['database_path']))
+            #loading a file
+            error = phreeqc.load_database("/Users/Andrew/Documents/Research/University of Victoria Civil Engineering/ROSSpy/rosspy/databases/unix/pitzer.dat")  
+            print(error)
+            print(phreeqc.get_error_string())
+    
+            # loading a string
+#             db = open(self.parameters['database_path'])
+#             database = '\n'.join([line for line in db])
+#             print(database)
+            phreeqc.load_database_string('solution_master_species')
+            print(dir(phreeqc))
+#             database = '\n'.join([line for line in db])
+            print(phreeqc.get_error_string())
+            
+            # run the simulation
             phreeqc.run_string(input_file)
             
             # define the conc dictionary
@@ -729,7 +779,6 @@ class ROSSPkg():
             for row in output[1:]:
                 for col, head in enumerate(header):
                     conc[head].append(row[col])
-                    
             return phreeqc, conc
 
         def main(input_file):
@@ -743,17 +792,15 @@ class ROSSPkg():
             phreeqc, conc, run_time = measure_time(run, input_file)            
             
             # export the simulation results
-            fobj = open(self.parameters['output_path'], 'w')
             headers = conc.keys()
             self.results['csv_data'] = pandas.DataFrame(conc, columns = headers)
+            self.results['csv_data'].to_csv(self.parameters['output_path'])
             if self.verbose:
                 if self.jupyter:
                     pandas.set_option('display.max_columns', None)
                     display(self.results['csv_data'])
                 else:
                     print(self.results['csv_data'])
-            fobj.write(self.results['csv_data'].to_string())
-            
             self.variables['run_time (s)'] = float(run_time)
 
         # communicate progress to the user
@@ -778,7 +825,6 @@ class ROSSPkg():
 
     def process_selected_output(self, selected_output_path = None, plot_title = None, title_font = 'xx-large', label_font = 'x-large', x_label_number = 6, export_name = None, export_format = 'svg', individual_plots = None):
         """Interpreting the PHREEQC SELECTED_OUTPUT file and conducting the plotting functions"""
-        databases = [re.search('(\w+)(?=.json)', database).group() for database in glob('./databases/*.json')]
         if plot_title is not None:
             self.plot_title = plot_title
         
@@ -798,7 +844,7 @@ class ROSSPkg():
 
                 # define the database contents
                 self.parameters['database_selection'] = 'pitzer'
-                for database in databases:
+                for database in self.databases:
                     names = database.split('_')
                     if all(re.search(name, selected_output_path, re.IGNORECASE) for name in names):
                         self.parameters['database_selection'] = database                   
@@ -816,7 +862,11 @@ class ROSSPkg():
 
             # preparing the SELECTED_OUTPUT file into a dataframe
             selected_output = open(selected_output_path, 'r')
-            original_data = pandas.read_table(selected_output, sep = '\t')
+            if re.search('pqo', selected_output_path):
+                sep = '\t'
+            else:
+                sep = ','
+            original_data = pandas.read_table(selected_output, sep = sep)
             self.results['csv_data'] = pandas.DataFrame(original_data)
             for column in self.results['csv_data'].columns:
                 new_column = column.strip()
@@ -969,24 +1019,26 @@ class ROSSPkg():
             for mineral in self.variables['precipitated_minerals']: 
                 mineral_formula = self.minerals[mineral]['formula']
                 legend.append(legend_determination(0, mineral, mineral_formula))
-                data[f'{mineral} (mol)'] = {}
+                data[f'{mineral} (g)'] = {}
                 cf_series = []        
                 scaling_series = []
                 for index, row in self.results['csv_data'].iterrows():
                     if index != len(self.results['csv_data']['mass_H2O']):
                         if row['step'] >= 1:
                             cf = self.variables['initial_solution_mass'] / row['mass_H2O']
-                            scale = row[mineral]
-                            scaling_series.append(scale) 
+                            scale_mass = row[mineral] * self.minerals[mineral]['mass']
+                            scaling_series.append(scale_mass) 
                             cf_series.append(cf)   
-                            data[f'{mineral} (mol)'][sigfigs_conversion(cf, 6)] = scale              
+                            data[f'{mineral} (g)'][sigfigs_conversion(cf, 6)] = scale_mass            
 
                 pyplot.plot(cf_series,scaling_series)
                 data_df = pandas.DataFrame(data)
                 scaling_data = scaling_data.merge(data_df, how = 'outer', left_index = True, right_index = True)
                     
+            legend_title = 'scale'
+            mineral = None
             if self.plot_title is None:
-                self.plot_title = 'Evaporation precipiation from the {}'.format(self.parameters['water_selection'])                    
+                self.plot_title = 'Evaporation scaling from the {}'.format(self.parameters['water_selection'])                    
             self.illustrate(pyplot, legend, legend_title, mineral, export_name, label_font, title_font, export_format)
             return scaling_data
         
@@ -1109,11 +1161,12 @@ class ROSSPkg():
             print('No scaling occurred.')
             return None            
 
-        # plot the simulation depending upon the simulation perspective       
+        # plot the simulation depending upon the simulation perspective   
+        self.parameters['individual_plots'] = individual_plots
         if self.parameters['simulation_type'] == 'evaporation':
             scaling_data = evaporation()
         elif self.parameters['simulation_perspective'] == "all_time":
-            if individual_plots is None:
+            if self.parameters['individual_plots'] is None:
                 self.parameters['individual_plots'] = True
             if self.parameters['individual_plots']:
                 scaling_data = all_time_plotting()
@@ -1123,7 +1176,7 @@ class ROSSPkg():
                 self.illustrate(pyplot, legend, legend_title, mineral, export_name, label_font, title_font, export_format)
 
         elif self.parameters['simulation_perspective'] == 'all_distance':
-            if individual_plots is None:
+            if self.parameters['individual_plots'] is None:
                 self.parameters['individual_plots'] = False    
             if not self.parameters['individual_plots']:
                 legend, scaling_data = all_distance_plotting()
@@ -1145,9 +1198,9 @@ class ROSSPkg():
         # determine the y-axis label
         if self.parameters['simulation'] == 'scaling':
             if self.parameters['simulation_type'] == 'transport':
-                y_label = 'Quantity (g/m^2)'
+                y_label = 'Mass concentration (g/m^2)'
             elif self.parameters['simulation_type'] == 'evaporation':
-                y_label = 'Quantity (moles)'
+                y_label = 'Mass (g)'
         elif self.parameters['simulation'] == 'brine':
             y_label = 'Concentration (molal)'
         
@@ -1169,28 +1222,25 @@ class ROSSPkg():
         pyplot.xlabel(x_label, fontsize = label_font)
         pyplot.ylabel(y_label, fontsize = label_font)  
         if legend is not None:
-            pyplot.legend(legend, title = legend_title, loc='best', title_fontsize = 'x-large', fontsize = 'large')   
-        pyplot.figtext(0.2, 0.07, 'Final CF: {}'.format(sigfigs_conversion(self.variables['simulation_cf'], 4)), wrap=True, horizontalalignment='left', fontsize=12)
+            pyplot.legend(legend, title = legend_title, loc='best', title_fontsize = 'x-large', fontsize = 'large')  
+        if self.parameters['simulation_type'] == 'transport':
+            pyplot.figtext(0.2, 0.07, 'Final CF: {}'.format(sigfigs_conversion(self.variables['simulation_cf'], 4)), wrap=True, horizontalalignment='left', fontsize=12)
         
         figure = pyplot.gcf()
         pyplot.show()
         self.export_plot(figure, mineral, export_name, export_format)
-        
-    def figure_name(self):
-        # define the output name
-        if self.parameters['simulation'] == 'scaling':
-            if self.parameters['individual_plots']:
-                export_name = mineral
-            else:
-                export_name = 'all_minerals'
-        if self.parameters['simulation'] == 'brine':
-            export_name = 'brine'
-        return export_name
 
     def export_plot(self, figure, mineral = None, export_name = None, export_format = 'svg'):
         """Export the plots to the current working directory  """
         if export_name is None:
-            export_name = self.figure_name()
+            # define the output name
+            if self.parameters['simulation'] == 'scaling':
+                if self.parameters['individual_plots']:
+                    export_name = mineral
+                else:
+                    export_name = 'all_minerals'
+            if self.parameters['simulation'] == 'brine':
+                export_name = 'brine'
         self.results['figures'][export_name] = {'figure':figure, 'title':self.plot_title}
                                 
         # export the plot
